/* Encyklopedie jazyka C
 *
 * Kapitola Příkazy jazyka
 *
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "3.8"
#define NAZEV "Příkazy jazyka C"

// TODO Prostuduj si obsah tohoto hlavičkového souboru! Jsou tam zaklínadla,
// bez kterých některé věci ve Windows nefungují správně!
// - barevný výstup
// - printg - český tisk (překódovává z UTF8 na CP852)
// - printh - český tisk + vyprávěcí mód
// - formátovací a formátové značky pro printf/scanf podle normy
// - obsluha nabídek v kapitolách
#include "pomocny-modul.h"

// Implementace nabídkového menu
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
// #include <string.h>
#include <stdbool.h>
#include <ctype.h>
#include <math.h>
// #include <float.h>
// #include <limits.h>


// Hlavičky funkcí realizující jednotlivé kapitoly.
// Implementace jsou za funkcí main.
void blokovyPrikaz(void);
void podprogram(void);

void podminenyPrikazIfElse(void);
void podminenyPrikazSwitch(void);

void prikazCykluWhile(void);
void prikazCykluFor(void);
void prikazCykluDoWhile(void);

void prikazReturn(void);
void prikazBreak(void);
void prikazContinue(void);
void prikazGoto(void);

void prikazDefinicePromenne(void);
void vyrazovyPrikaz(void);


/** Startovní bod programu */
int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "==============================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2021--2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "==============================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printg("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }

  menuPridej(menu,   0, "\n" SEKCE("Blokové příkazy") , NULL);
  menuPridej(menu, 'A', "Blokový příkaz {}\n", blokovyPrikaz);
  menuPridej(menu, 'B', "Podprogram\n", podprogram);

  menuPridej(menu,   0, "\n" SEKCE("Podmíněné příkazy") , NULL);
  menuPridej(menu, 'C', "if-else\n", podminenyPrikazIfElse);
  menuPridej(menu, 'D', "switch\n", podminenyPrikazSwitch);

  menuPridej(menu,   0, "\n" SEKCE("Příkazy cyklu") , NULL);
  menuPridej(menu, 'E', "while\n", prikazCykluWhile);
  menuPridej(menu, 'F', "for\n", prikazCykluFor);
  menuPridej(menu, 'G', "do-while\n", prikazCykluDoWhile);

  menuPridej(menu,   0, "\n" SEKCE("Příkazy skoků") , NULL);
  menuPridej(menu, 'H', "return\n", prikazReturn);
  menuPridej(menu, 'I', "break\n", prikazBreak);
  menuPridej(menu, 'J', "continue\n", prikazContinue);
  menuPridej(menu, 'K', "goto\n", prikazGoto);

  menuPridej(menu,   0, "\n" SEKCE("Specifické příkazy") , NULL);
  menuPridej(menu, 'L', "Definice proměnné\n", prikazDefinicePromenne);
  menuPridej(menu, 'M', "Výrazy a výrazové příkazy\n", vyrazovyPrikaz);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}




////////////////////////////////////////////////////////////////////////////////
void blokovyPrikaz(void)
{
  zacatekKapitoly("Blokový příkaz");

  printh(
    "- Umožňuje realizovat sekvenci více příkazů v místech,\n"
    "  kde se očekává jediný příkaz (podmínky, cykly).\n"
    "\n"
    SEKCE("Syntaxe")
    "\n"
    KOD(
    "  { <posloupnost příkazů> }\n"
    )
    "\n"
    "- " KK("<posloupnost příkazů>") " může být i prázdná\n"
    "\n"
  );

  if (navratNeboSekce("Výroba lokální proměnné uvnitř bloku")) return;
  if (pokracovat())
  {
    /* Blokový příkaz může fungovat i samostatně,
       tedy bez cyklu nebo podmíněného příkazu. Užitečný může být například
       v příkazu switch - viz dále.

       Blokový prikaz může mít své lokální proměnné, které zanikají na konci
       bloku.

       Za blokovým příkazem se nepíše středník.
     */
    printh(
      "- Blokový příkaz může mít své vlastní lokální proměnné.\n"
      "- Na konci bloku jeho lokální proměnné zanikají.\n"
      "\n"
      KOD(
      "  {\n"
      )
    );

    {
      int cislo = 222;
      printh(
        KOD(
        "    int cislo = %d;\n"
        )
        , cislo
      );
    }
//    cislo = 333; // Odkomentuj řádek a zkus to.

    printh(
      KOD(
      "  }\n"
      "  cislo = 333; // syntaktická chyba, proměnná právě zanikla\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Lokální proměnné cyklu")) return;
  if (pokracovat())
  {
    printh(
      "- Lokální proměnnou jde vytvořit i v bloku který je tělem cyklu.\n"
      "- Hodnota lokální proměnné bloku nepřežije do další iterace.\n"
      "- Přežijí jen proměnné definované v hlavičce cyklu " KK("for") ".\n"
      "\n"
      KOD(
      "  for(int i = 0; i < 5; ++i)\n"
      "  {\n"
      "    int x = 123;\n"
      "    printf(\"%%d \", x);\n"
      "    x += 111; // tohle teď nemá význam\n"
      "  }\n"
      )
      "\n"
      "  => "
    );

    for(int i = 0; i < 5; ++i)
    {
      int x = 123;
      printf("%d ", x);
      x += 111; // tohle teď nemá význam
    }
    printh(
      "\n"
      "\n"
      "- Jak proměnná " KK("x") ", tak proměnná " KK("i") " po vykonání cyklu zaniknou.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void podprogram(void)
{
  zacatekKapitoly("Podprogram");

  printh(
    "- Dá se považovat za zobecnění blokového příkazu. Rozlišuje se ale\n"
    "  definice podprogramu a volání podprogramu.\n"
    "=> Tělem cyklu může být příkaz volání podprogramu, ne jeho definice\n"
    "   s hlavičkou a tělem.\n"
    "\n"
    "- Blíže viz kapitola Funkce. Zde jen velice stručně.\n"
    "\n"
  );

  if (navratNeboSekce("Rozhraní podprogramu")) return;
  if (pokracovat())
  {
    printh(
      "- Podprogram je relativně samostatný pojmenovaný kus kódu, který s okolním\n"
      "  programem komunikuje přes své rozhraní, které tvoří\n"
      "  - " BB("(formální) parametry") " - vstupní proměnné deklarované v hlavičce, kterými\n"
      "    proudí informace dovnitř podprogramu,\n"
      "  - " BB("návratová hodnota funkce") " - výsledná hodnota vracená z funkcí příkazem\n"
      "    return,\n"
      "  - " BB("argumenty (skutečné parametry)") " - hodnoty dosazované při volání do\n"
      "    (formálních) parametrů.\n"
      "\n"
      "- " BB("Funkce") " je podprogram, který vrací hodnotu určitého typu.\n"
      "- " BB("Procedura") " je podprogram, který žádnou hodnotu nevrací. V hlavičce používá\n"
      "  návratový typ " KK("void") ".\n"
      "\n"
    );
  }

  if (navratNeboSekce("Syntaxe volání podprogramu")) return;
  if (pokracovat())
  {
    printh(
      KOD(
      "  procedura(<seznam argumentů>);\n"
      "  proměnná = funkce(<seznam argumentů>);\n"
      )
      "\n"
      "- Volání procedury představuje (zobecněný) krok programu.\n"
      "- Volání funkce představuje výpočet s výslednou hodnotou, kterou\n"
      "  lze získat a dále použít v jiném výrazu (např. přiřazení do proměnné).\n"
      "- Kulaté závorky za jménem podprogramu nelze vynechat.\n"
      "  => Jméno podprogramu bez závorek je ukazatel na funkci - viz kapitoly\n"
      "     o funkcích a ukazatelích.\n"
      "\n"
      PODSEKCE("Seznam argumentů")
      "- Argumenty se oddělují čárkami.\n"
      "- Počet, pořadí a typy argumentů musí odpovídat počtu, pořadí a typům\n"
      "  parametrů, které jsou deklarovány v hlavičce daného podprogramu.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Příklad volání podprogramů")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  printf(\"Odmocnina z %%g je %%g.\\n\", 15.4, sqrt(15.4));\n"
      )
      "  => "
    );

    printf("Odmocnina z %g je %g.\n", 15.4, sqrt(15.4));

    printh(
      "\n"
      "- Funkce " KK("sqrt") " zpracuje svůj argument a vrací výslednou hodnotu, která\n"
      "  je zde rovnou použita jako poslední argument pro " KK("printf") ".\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void podminenyPrikazIfElse(void)
{
  zacatekKapitoly("Podmíněný příkaz if-else");

  printh(
    "- Vyhodnotí logický výraz a provede jednu nebo druhou větev.\n"
    "- První větev se provede, když je logický výraz pravdivý.\n"
    "- Větev za klíčovým slovem " KK("else") " se provede, když je logický výraz nepravdivý.\n"
    "- Větev " KK("else") " nemusí být přítomna.\n"
    "\n"
  );

  if(navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  if ( <logický výraz> ) <příkaz>\n"
      "  if ( <logický výraz> ) <příkaz> else <příkaz>\n"
      )
      "\n"
      PODSEKCE("Doporučená syntaxe")
      "\n"
      KOD(
      "  if ( <logický výraz> )\n"
      "  {\n"
      "    <posloupnost příkazů>\n"
      "  }\n"
      "  else\n"
      "  {\n"
      "    <posloupnost příkazů>\n"
      "  }\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka použití")) return;
  if (pokracovat())
  {
    printh(
      PODSEKCE("Plná varianta s větví else")
      "\n"
      KOD(
      "  if (1 > 2)\n"
      "  { printf(\"Podmínka je pravdivá.\\n\"); }\n"
      "  else\n"
      "  { printf(\"Podmínka není pravdivá.\\n\"); }\n"
      )
      "\n"
      "  =>"
    );

    if (1 > 2)
    { printg("Podmínka je pravdivá.\n"); }
    else
    { printg("Podmínka není pravdivá.\n"); }

    printf("\n");
  }

  if (navratNeboSekce("Větev else jde vynechat")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      "- Pozor ale na formátování!\n"
      "\n"
      KOD(
      "  if (1 > 2)\n"
      "    printf(\"Podmínka je pravdivá.\\n\");\n"
      "    printf(\"Toto se provede vždy - toto je příkaz za podmíněným příkazem.\\n\");\n"
      )
      "\n"
      "- Tady může překladač vypisovat varování.\n"
      "- Bezpečnější je používat vždy jako tělo větve blokový příkaz.\n"
      "\n"
    );

// Vypínám varování/chybové hlášení, protože chci tuto chybu demonstrovat.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
    if (1 > 2)
      printg("  =>Podmínka je pravdivá.\n");
      printg("  =>Toto se provede vždy - toto je příkaz za podmíněným příkazem.\n");
    // Tady pozor na odsazování! I kdyby byl druhý printg odsazený, pořád nebude patřit do těla příkazu if!
#pragma GCC diagnostic pop

    printh(
      "\n"
    );
  }

  if (navratNeboSekce("Zřetězený if")) return;
  if (pokracovat())
  {
    /* Větev s více příkazy musí tvořit blokový příkaz.

    Pozor! nesmí se za ním napsat středník, jinak by se bral jako příkaz za
    příkazem if a případná větev else by byla syntaktickou chybou.

    Je dobrým zvykem používat blokový příkaz i v případech, kdy větev tvoří
    jediný příkaz. Je to čitelnější. V budoucnu je pak snazší přidávat další
    příkazy.

    Příkaz if může sám o sobě tvořit větev jiného příkazu if.
    */
    printh(
      "- Příkaz " KK("if") " může být tělem jiného příkazu " KK("if") ".\n"
      "- Příkazy " KK("if") " lze takto zřetězit.\n"
      "- Konstrukce připomíná příkaz " KK("switch") ", ale umožňuje vyhodnocovat složitější\n"
      "  podmínky.\n"
      "\n"
      "  Zadej znak a stiskni Enter\n"
      "\n"
      KOD(
      "  char znak = "
      )
    );
    char znak = getchar();
    if (znak != '\n') pozerKonecRadku();

    printh(
      KOD(
      "  if (znak >= 'a' && znak <= 'z')\n"
      "  {\n"
      "    printf(\"Znak %%c je malé písmeno. Velké vypadá takto: %%c\\n\", znak, toupper(znak));\n"
      "  }\n"
      "  else if (znak >= 'A' && znak <= 'Z')\n"
      "  {\n"
      "    printf(\"Znak %%c je velké písmeno. Malé vypadá takto: %%c\\n\", znak, tolower(znak));\n"
      "  }\n"
      "  else\n"
      "  {\n"
      "    printf(\"Přečtený znak (%%c, ASCII kód %%d) není písmeno!\\n\", znak, znak);\n"
      "  }\n"
      )
      "\n"
      "  =>"
    );

    if (znak >= 'a' && znak <= 'z')
    {
      printg("Znak %c je malé písmeno. Velké vypadá takto: %c\n", znak, toupper(znak));
    }
    else if (znak >= 'A' && znak <= 'Z') // tento if je větví else předchozího if
    {
      printg("Znak %c je velké písmeno. Malé vypadá takto: %c\n", znak, tolower(znak));
    }
    else // toto je větev else druhého příkazu if
    {
      printg("Přečtený znak (%c, ASCII kód %d) není písmeno!\n", znak, znak);
    }

    printg("\n");
  }

  if (navratNeboSekce("Hodnota jako logický výraz")) return;
  if (pokracovat())
  {
    /* Hodnotu jakéhokoli typu lze brát i jako logickou hodnotu!

    Platí, že nulová hodnota se vyhodnotí jako nepravda,
    kdežto jakákoli nenulová hodnota se vyhodnotí jako pravda
    */
    printh(
      "- Jakákoli hodnota může být považována za logickou hodnotu.\n"
      "- Výraz se v podmínce přetypuje na " KK("int") ".\n"
      "- Platí, že nula je nepravda, cokoli jiného je pravda.\n"
      "\n"
    );

    if (-10) printh("  " KK("if (-10)") " => vyhodnotí se jako pravda\n");
    else     printh("  " KK("if (-10)") " => vyhodnotí se jako nepravda\n");

    if (0) printh("  " KK("if (0)") " => vyhodnotí se jako pravda\n");
    else   printh("  " KK("if (0)") " => vyhodnotí se jako nepravda\n");

    // Znak je obvykle pravda, pouze znak '\0' je nepravda.
    if ('n') printh("  " KK("if ('n')") " => vyhodnotí se jako pravda\n");
    else     printh("  " KK("if ('n')") " => vyhodnotí se jako nepravda\n");

    // Znak je obvykle pravda, pouze znak '\0' je nepravda.
    if ('\0') printh("  " KK("if ('\\0')") " => vyhodnotí se jako pravda\n");
    else      printh("  " KK("if ('\\0')") " => vyhodnotí se jako nepravda\n");

    /* U textového řetězce se zde vyhodnotí nikoli obsah, ale adresa, na které
    text leží => něco takového se prakticky vždy vyhodnotí jako pravda.
    */
    if ("nepravda") printh("  " KK("if (\"nepravda\")") " => vyhodnotí se jako pravda\n");
    else            printh("  " KK("if (\"nepravda\")") " => vyhodnotí se jako nepravda\n");

    // Taky pravda
    if ("") printh("  " KK("if (\"\")") " => vyhodnotí se jako pravda\n");
    else    printh("  " KK("if (\"\")") " => vyhodnotí se jako nepravda\n");

    // Tohle je prázdná adresa - typicky je nulová => nepravda
    if (NULL) printh("  " KK("if (NULL)") " => vyhodnotí se jako pravda\n");
    else      printh("  " KK("if (NULL)") " => vyhodnotí se jako nepravda\n");

    printg("\n");
  }

  if (navratNeboSekce("Časté chyby")) return;
  if (pokracovat())
  {
    /* Výsledek přiřazení má hodnotu - a ta se dá vyhodnotit jako logická hodnota!

    Překladač v tomto případě tiskne varování - věnujte mu pozornost.

    Pokud to je finta, kterou opravdu chcete udělat, přidejte jedny závorky navíc:
    if ((a = 100))
    */
    printh(
      "- Pozor na záměnu = a ==\n"
    );

// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wparentheses"
    float a = 10.5;
    if (a = 100) // varování překladače: suggest parentheses around assignment used as truth value
      printh(
        "  " KK("if (a = 100)") " => Pravda! Dojde k přiřazení a = %g a nenulová hodnota\n"
        "                  je pravda.\n"
        , a
      );

    if (a == 100)
      printh(
        "  " KK("if (a == 100)") " => Teď se porovnává. Výsledek je skutečně pravda,\n"
        "                   protože v proměnné a teď je hodnota %g.\n"
        , a
      );


    /* Říkáme "nerovná se", nikoli "rovná se ne".
       Výraz a =! 0 znamená a = (!0), tedy do a přiřaď logickou negaci nuly.
       Operátor ! je logický operátor NOT.
    */
    printh(
      "\n"
      "- Pozor na chybné nerovnítko. Toto =! není toto !=.\n"
    );
    if (a =! 0) // varování překladače: suggest parentheses around assignment used as truth value
      printh(
        "  " KK("if (a =! 0)") " => Teď se do " KK("a") " přiřadila logická negace hodnoty 0,\n"
        "                 což je %g, a tedy " KK("true") ".\n"
        "                 Operátor " KK("!") " je logický operátor NOT.\n"
        "\n"
        , a
    );
#pragma GCC diagnostic pop
  }

  if (navratNeboSekce("Problém s porovnáváním desetinných čísel")) return;
  if (pokracovat())
  {
    /* Tohle není problém jazyka C, ale problém s ukládáním desetinných čísel
    v počítači.

    Typy float a double ukládají čísla s různou přesností, tj. zachytí různě
    dlouhé počty desetinných míst (float asi 7, double asi 15, když to bereme
    v desítkové soustavě).

    Hodnota 0.1 nejde na počítači uložit přesně, protože ve dvojkové soustavě
    je to periodické číslo.

    Konstatna 0.2 použitá ve výrazu je automaticky typu double.

    Závěr - pro desetinná čísla raději používejte neostré rovnosti <=, >=.
    */
    printh(
      "- Ostrá rovnost u desetinných čísel není dobrý nápad!\n"
      "- Porovnávání hodnot typů " KK("float") " a " KK("double") " může vést k chybám.\n"
      "- Blíže viz kapitola Číselné typy.\n"
      "\n"
      KOD(
      "  float a = 0.1; // Uloží se to nepřesně! Konstanta 0.1 je typu double.\n"
      )
    );
    float a = 0.1; // uloží se to nepřesně!
    if (a == 0.1)
      printh("  " KK("if (a == 0.1)") " => pravda, a = %.15f\n", a);
    else
      printh("  " KK("if (a == 0.1)") " => nepravda!!!, a = %.15f\n", a);

    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void podminenyPrikazSwitch(void)
{
  zacatekKapitoly("Podmíněný příkaz switch");

  printh(
    "- Vyhodnotí celočíselný výraz a skočí na návěští " KK("case") ", se shodnou\n"
    "  hodnotou.\n"
    "- Pokud hodnota žádného z návěští neodpovídá, skočí na (nepovinné)\n"
    "  návěští " KK("default") ".\n"
    "- Pokud návěští " KK("default") " není použito, příkaz neprovede žádný kód a končí.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  switch ( <celočíselný výraz> )\n"
      "  {\n"
      "    case <celočíselná konstanta>:\n"
      "      <posloupnost příkazů>\n"
      "    case <celočíselná konstanta>:\n"
      "      <posloupnost příkazů>\n"
      "    default:\n"
      "      <posloupnost příkazů>\n"
      "  }\n"
      )
      "\n"
      "- <Posloupnost příkazů> za návěštím by měla končit příkazem " KK("") ".\n"
      "- Pokud nekončí, pokračuje se posloupností příkazů za dalším návěštím.\n"
      "- Někdy je toto chování zamýšlené.\n"
      "\n"
    );
  }

  /* Příkaz switch v jazyce C umí rozlišovat pouze celočíselné hodnoty.

  V návěštích case má smysl používat pouze celočíselné nebo znakové konstanty.

  Pro specifikaci návěští case jde použít pouze konstanty, ne proměnné.

  Návěští samo není žádným příkazem. Je to jen označení místa, kam příkaz switch
  může skočit.

  Posloupnost příkazů za každým návěštím může být prázdná.

  Typicky posloupnost příkazů za návěštím končí příkazem break. Pokud příkaz
  break chybí, bude program pokračovat dalšími příkazy za dalším návěštím.

  Pozor! Používat pouze proměnné vyrobené vně příkazu switch!
  Je-li potřeba pro kód za návěštím vytvořit lokální proměnnou, je vhodné
  použít blokový příkaz. Jinak jsou lokální proměnné vyráběné uvnitř příkazu
  switch zdrojem nepříjemných chyb (neinicializované hodnoty, mylné opakované
  použití aj.).
  */

  if (navratNeboSekce("Ukázková úloha s příkazem switch")) return;
  if (pokracovat())
  {
    printh(
      "- Kód viz zdrojový text.\n"
      "\n"
    );

// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
    int konec = 0; // false
    while (!konec)
    {
      printg("Zadej jedno z písmen a, b, c, nebo písmeno k pro konec: ");
      int volba = prvniPismenoNaRadku();
      switch (volba)
      {
      //  case cislo: // syntaktická chyba
        case 'a':
          printg("Zadal jsi volbu a\n");
          // tady chybí break, proto se pokračuje dalším příkazem
          // při zapnutém -Wextra vypisuje překladač varování
        case 'b':
          printg("Zadal jsi volbu a nebo b.\n");
          break;
        default: // vetev default nemusi byt nutne na konci
          printg("Pro volbu '%c' nic specifického neumím => větev default.\n", volba);
          break;
        case 99: // konstanta 'c' má hodnotu 99 -> kód znaku c v ASCII tabulce
          printg("Zadal jsi volbu c.\n");
          break;
        case 'k':
          printg("Končím cyklus.\n");
          konec = 1; // true
          break;
      }
    }
#pragma GCC diagnostic pop

    printh(
      "\n"
    );
  }

  if (navratNeboSekce("Lokální proměnné v těle příkazu switch")) return;
  if (pokracovat())
  {
    printh(
      "- Pozor! Používat pouze proměnné vyrobené vně příkazu " KK("switch") "!\n"
      "- Je-li potřeba pro kód za návěštím vytvořit lokální proměnnou, je vhodné\n"
      "  použít blokový příkaz.\n"
      "- Blíže viz zdrojový text.\n"
      "\n"
      PODSEKCE("Ukázka")
      "\n"
      KOD(
      "  int x;\n"
      "  switch (volba) {\n"
      "    case 'a':\n"
      "      int x = 666; // syntaktická chyba!\n"
      "      printf(\" => volba: %%d, x: %%d\\n\", volba, x);\n"
      "      break;\n"
      "    case 'c':\n"
      "      {\n"
      "        int x = 111; // x je lokální v tomto bloku a překrývá druhou proměnnou x\n"
      "        printf(\" => volba: %%d, x: %%d\\n\", volba, x);\n"
      "      }\n"
      "      break;\n"
      "  }\n"
      )
      "\n"
    );


    int x = 333;
    int konec = 0; // false
    while (!konec)
    {
      printg("Zadej jedno z písmen a, b, c, nebo něco jiného pro konec:");
      int volba = prvniPismenoNaRadku();
      switch (volba)
      {
        case 'a':
          //int x = 666; // syntaktická chyba
          printf(" => volba: %d, x: %d\n", volba, x);
          break;
        case 'b':
          printf(" => volba: %d, x: %d\n", volba, x);
          break;
        case 'c':
          {
            int x = 111; // x je lokální v tomto bloku a překrývá druhou proměnnou x
            printf(" => volba: %d, x: %d\n", volba, x);
          }
          break;
        default:
          konec = 1;
          break;
      }
    }

    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazCykluWhile(void)
{
  zacatekKapitoly("Příkaz cyklu while");

  printh(
    "- Cyklus s podmínkou na začátku => tělo se nemusí vykonat ani jednou.\n"
    "- Vykoná další iteraci, když je podmínka pravdivá.\n"
    "- Když je podmínka nepravdivá, pokračuje kód za tělem cyklu.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  while ( <logický výraz> ) <příkaz>\n"
      )
      "\n"
      PODSEKCE("Doporučená syntaxe")
      "\n"
      KOD(
      "  while ( <logický výraz> )\n"
      "  {\n"
      "    <posloupnost příkazů>\n"
      "  }\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Sémantika - význam")) return;
  if (pokracovat())
  {
    /* Je to cyklus vyhodnocující podmínku na začátku.

    Funguje takto: if (podmínka) -> tělo -> if (podmínka) -> tělo -> ...

    Tělo se vykoná jen tehdy, když se podmínka vyhodnotí jako pravdivá. Když
    se vyhodnotí jako nepravdivá příkaz končí a pokračuje se dalším příkazem.

    Podmínka v kulatých závorkách nesmí být nikdy prázdná.

    Nemusí proběhnout ani jediná iterace - to když je podmínka nepravdivá už
    na začátku.

    Tělo cyklu tvoří jediný příkaz - silně doporučuji používat vždy blokový příkaz.
    */
    printh(
      "\n"
      KOD(
      "  while (<1>) <2>\n"
      )
      "\n"
      "- Vykonává posloupnost kroků v pořadí: <1>, <2>, <1>, <2>, <1>, ...\n"
      "\n"
      "- Podmínka <1> nesmí být prázdná. Musí jít o výraz, jehož hodnota jde\n"
      "  přetypovat na " KK("int") " a vyhodnotit jako logickou hodnotu.\n"
      "\n"
      "- Jakmile se podmínka <2> vyhodnotí jako nepravdivá, cyklus končí.\n"
      "- Tělo <2> může být prázdný příkaz (" KK(";") " nebo " KK("{}") "), nelze jej ale vynechat.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka cyklu while - 5 iterací")) return;
  if (pokracovat())
  {
    printh(
      "- Cyklus while od 0 do 4 => 5 iterací\n"
      "\n"
      KOD(
      "  int i = 0;\n"
      "  while (i < 5)\n"
      "  {\n"
      "    printf(\"  %%d\", i);\n"
      "    i += 1;\n"
      "  }\n"
      )
      "\n"
      "  =>"
    );
    int i = 0;
    while (i < 5)
    {
      printg("  %d", i);
      i += 1; // i = i + 1
    }

    printg("\n\n");
  }

  if (navratNeboSekce("Tisk hodnot oddělených čárkami")) return;
  if (pokracovat())
  {
    printh(
      "- Tisk hodnot oddělených čárkami tak, aby za poslední čárka nebyla...\n"
      "\n"
      KOD(
      "  int i = 1;\n"
      "  printf(\"%%d\", i++);\n"
      "  while (i <= 5)\n"
      "  {\n"
      "    printf(\", %%d\", i++);\n"
      "  }\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );

    int i = 1;
    printf("%d", i++);
    while (i <= 5)
    {
      printf(", %d", i++); // i++ vrací původní hodnotu a až potom přičte jedničku
    }
    printf("\n\n");
  }

  if (navratNeboSekce("Pozor na tělo bez blokového příkazu")) return;
  if (pokracovat())
  {
    /* Když je podmínka vyhodnocena jako nepravdivá hned na začátku,
    tělo cyklu se nikdy neprovede.

    Pozor na tělo s jedním příkazem - při nevhodném formátování může další příkaz
    vypadat, jako kdyby patřil do těla cyklu. Doporučuji používat vždy blokový
    příkaz {}. Překladač tady tiskne varování (když je zapnut přepínač -Wextra).
    */
    printh(
      "- Pozor na tělo bez blokového příkazu. Všimni si nevhodného formátování.\n"
      "- Překladač může tisknout varování.\n"
      "- Cyklus " KK("while (0)") " => podmínka je " KK("false") " => ani jedna iterace\n"
      "\n"
      KOD(
      "  while (0)\n"
      "    printf(\"Tohle se nikdy neprovede.\\n\");\n"
      "    printf(\"Tohle už není tělo cyklu! Je to další příkaz => provede se vždy!\\n\");\n"
      )
      "\n"
      "  =>"
    );
// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
    while (0) // varování překladače: this ‘while’ clause does not guard...
      printg("Tohle se nikdy neprovede.\n");
      printg("Tohle už není tělo cyklu! Je to další příkaz => provede se vždy!\n");

#pragma GCC diagnostic pop

    printg("\n");
  }

  if (navratNeboSekce("Cyklus s kódem v podmínce")) return;
  if (pokracovat())
  {
    printh(
      "- Cyklus " KK("while") " s kódem v podmínce a prázdným tělem\n"
      "- Používá se často při zpracovávání souborů a datových proudů.\n"
      "\n"
      KOD(
      "  printf(\"Zadej libovolný počet mezer, pak znak a pak Enter.\\n\");\n"
      "  int znak; // opravdu int, protože getchar vrací hodnotu typu int\n"
      "  while ( ( znak = getchar() ) == ' ')\n"
      "  { /*prázdné tělo*/ }\n"
      )
      "\n"
      "  =>"
    );
    printg("Zadej libovolný počet mezer, pak znak a pak Enter.\n  =>");
    int znak; // opravdu int, protože getchar vrací hodnotu typu int
    while ( ( znak = getchar() ) == ' ')
    { /*prázdné tělo*/ }

    if (znak == '\n')
      printg("  Přeskákal jsem mezery, ale nic jsi nezadal.\n");
    else if (znak == EOF)
      printg("  Ukončil jsi vstup.\n");
    else
      printg("  První znak, co není mezera je '%c' (ASCII kód:%d).\n", znak, znak);

    pozerKonecRadku();
    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazCykluFor(void)
{
  zacatekKapitoly("Příkaz cyklu for");

  printh(
    "- Cyklus s podmínkou na začátku => tělo se nemusí vykonat ani jednou.\n"
    "- Vykoná další iteraci, když je podmínka pravdivá (nebo prázdná).\n"
    "- Když je podmínka nepravdivá, pokračuje kód za tělem cyklu.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  for (<výraz>; <logický výraz>; <výraz>) <příkaz>\n"
      "  for (<definice lokální proměnné s inicializací> <logický výraz>; <výraz>) <příkaz>\n"
      // Poznámka: Středník je součástí definice, proto se do takového
      // schématického popisu nepíše - ve skutečnosti tam ale vždy bude.
      "  for ( ; ; ) <příkaz>\n"
      )
      "\n"
      PODSEKCE("Doporučená syntaxe")
      "\n"
      KOD(
      "  for (<výraz nebo definice>; <logický výraz>; <výraz>)\n"
      "  {\n"
      "    <posloupnost příkazů>\n"
      "  }\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Sémantika - význam")) return;
  if (pokracovat())
  {
    /* Je to cyklus skládající se z těchto částí:

    for (init; podmínka; výraz) tělo

    Funguje takto: init -> if (podmínka) -> tělo -> výraz -> if (podmínka) -> tělo -> výraz...

    V části init se obvykle vytváří a inicializuje řídící proměnná cyklu. Tato
    část se vykonává jen jednou a může zůstat prázdná.

    Tělo se vykoná jen tehdy, když se podmínka vyhodnotí jako pravdivá. Když
    se vyhodnotí jako nepravdivá příkaz končí a pokračuje se dalším příkazem.

    Když je podmínka prázdná, vždy se vyhodnotí jako pravdivá -> nekonečný cyklus.

    Po provedení těla se vyhodnotí výraz. Obvykle se tady mění řídící proměnná
    cyklu (++i, i--, i += 2, i *= 2 atd.). Tato část může zůstat prázdná.
    Poznámka: ++i je o chlup rychlejší než i++

    Nemusí proběhnout ani jediná iterace - to když je podmínka nepravdivá už
    na začátku.

    Tělo cyklu tvoří jediný příkaz - silně doporučuji používat vždy blokový příkaz.
    */
    printh(
      "\n"
      KOD(
      "  for (<1>; <2>; <3>) <4>\n"
      )
      "\n"
      "- Vykonává posloupnost kroků v pořadí: \n"
      "  <1>, <2>, <4>, <3>, <2>, <4>, <3>, <2>, <4>, <3>, ...\n"
      "- Části <1>, <2>, <3> mohou být prázdné.\n"
      "\n"
      "- Inicializační část <1> se vykoná jen jednou na začátku cyklu.\n"
      "\n"
      "- Je-li podmínka <2> prázdná, považuje se za pravdivou => nekonečný cyklus.\n"
      "- Jakmile se podmínka <2> vyhodnotí jako nepravdivá, cyklus končí.\n"
      "\n"
      "- Část <3> se vykoná vždy před vyhodnocením podmínky <2>.\n"
      "- Část <3> nejde přeskočit. Příkaz " KK("continue") " skáče na část <3>, ne na podmínku.\n"
      "- Část <3> typicky slouží pro změnu řídící proměnné cyklu, ale není to povinné.\n"
      "\n"
      "- Tělo <4> může být prázdný příkaz (" KK(";") " nebo " KK("{}") "), nelze jej ale vynechat.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka cyklu for - 5 iterací")) return;
  if (pokracovat())
  {
    printh(
      "- Cyklus for od 0 do 4 => 5 iterací\n"
      "\n"
      KOD(
      "  for (int i = 0; i < 5; ++i)\n"
      "  {\n"
      "    printf(\"  %%d\", i);\n"
      "  }\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );
    for (int i = 0; i < 5; ++i)
    {
      printg("  %d", i);
    }
    printg("\n");

    printg("\n");
  }

  if (navratNeboSekce("Tisk hodnot oddělených čárkami")) return;
  if (pokracovat())
  {
    printh(
      "- Tisk hodnot oddělených čárkami tak, aby za poslední čárka nebyla...\n"
      "\n"
      KOD(
      "  int i = 1;\n"
      "  printf(\"%%d\", i++);\n"
      "  for ( ; i <= 5; i++)\n"
      "  {\n"
      "    printf(\", %%d\", i);\n"
      "  }\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );
    int i = 1; // tady už proměnná i z minulého cyklu neexistuje!
    printg("%d", i++);
    for ( ; i <= 5; i++)  // kteroukoli část v hlavičce lze vynechat
    {
      printg(", %d", i);
    }
    printg("\n");

    printg("\n");
  }

  if (navratNeboSekce("Pozor na středník za hlavičkou")) return;
  if (pokracovat())
  {
    /* Pozor na omylem zapsaný středník za hlavičkou cyklu.
    Středník zde znamená prázdný příkaz použitý jako tělo.
    Blok, který vypadá jako tělo cyklu už k cyklu vůbec nepatří.
    */

    printh(
      "- Častý omyl: středník za hlavičkou => znamená prázdné tělo\n"
      "\n"
      KOD(
      "  for (int i = 0; i < 500; ++i); // tady je tělem středník!\n"
      "  {\n"
      "    printf(\"Tohle není tělo cyklu.\\n\");\n"
      "    printf(\"Tento celý blok se vykoná vždy právě jednou!\\n\");\n"
      "  }\n"
      )
      "\n"
    );

// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmisleading-indentation"
    for (int i = 0; i < 500; ++i); // tady je tělem středník!
  //^ Varování překladače: this ‘for’ clause does not guard...
    {
  //^ ...this statement, but the latter is misleadingly indented as if it were guarded by the ‘for’
      printg("  =>Tohle není tělo cyklu.\n");
      printg("  =>Tento celý blok se vykoná vždy právě jednou!\n");
    }
#pragma GCC diagnostic pop

    printg("\n");
  }

  if (navratNeboSekce("For s prázdnou podmínkou")) return;
  if (pokracovat())
  {
    /* Prázdná podmínka cyklu for se vyhodnotí jako pravdivá. */

    printh(
    "- For s prázdnou podmínkou => nekonečný cyklus\n"
      "\n"
      KOD(
    "  for (int i = 5; ; --i)\n"
    "  {\n"
    "    printf(\"All work and no play makes Jack a dull boy.\\n\");\n"
    "    if (i == 0) // Podmínka konce cyklu uvnitř těla je většinou prasárna!\n"
    "      break;\n"
    "  }\n"
      )
    "\n"
    );
    for (int i = 5; ; --i) // "nekonečný" cyklus
    {
      printf("  =>All work and no play makes Jack a dull boy.\n");
      if (i == 0) // Podmínka konce cyklu uvnitř těla je většinou prasárna!
        break;
    }

    printg("\n");
  }

  if (navratNeboSekce("Cyklus s více řídícími proměnnými")) return;
  if (pokracovat())
  {
    /* Cyklus nemusí mít pouze jedinou řídící proměnnou, může jich mít více.

    Pro tyto účely jde v posledním výrazu hlavičky použít operátor čárka (,). Ten
    umožní vyhodnotit v pořadí zleva doprava dva výrazy. Používá se téměř výhradně
    v cyklu for pro tento účel.
    */

    printh(
      "- Cyklus může mít více řídících proměnných\n"
      "- Pozor! Neznamená to, že jde o vícenásobný cyklus. Ten se dělá pomocí\n"
      "  vnořených cyklů.\n"
      "\n"
      KOD(
      "  for (int m = 1, a = 5, b = 24; a <= b; ++m, ++a, --b)\n"
      "  {\n"
      "    printf(\"%%*d %%*d\\n\", m, a, b-a+2, b);\n"
      "  }\n"
      )
    );
    for (int m = 1, a = 5, b = 24; a <= b; ++m, ++a, --b)
    {
      printf("  =>%*d %*d\n", m, a, b-a+2, b);
      // značka %*d -> za * se dosadí šířka (m), za d se dosadí hodnota (a)
    }
    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazCykluDoWhile(void)
{
  zacatekKapitoly("Příkaz cyklu do-while");

  printh(
    "- Cyklus s podmínkou na konci => tělo se vykoná nejméně jednou.\n"
    "- Vykoná další iteraci, když je podmínka pravdivá.\n"
    "- Když je podmínka nepravdivá, pokračuje kód za tělem cyklu.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  do příkaz while ( <logický výraz> );\n"
      )
      "\n"
      PODSEKCE("Doporučená syntaxe")
      "\n"
      KOD(
      "  do {\n"
      "    <posloupnost příkazů>\n"
      "  } while ( <logický výraz> ); \n"
      "//^                          ^\n"
      "//Začít závorkou             a končit středníkem!\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Sémantika - význam")) return;
  if (pokracovat())
  {
    /* Je to cyklus vyhodnocující podmínku na konci.

    Funguje takto: tělo -> if (podmínka) -> tělo -> if (podmínka) -> tělo -> ...

    Tělo se vykoná vždy nejméně jednou.

    Další iterace se vykoná jen tehdy, když se podmínka vyhodnotí jako pravdivá.
    Když se vyhodnotí jako nepravdivá příkaz končí a pokračuje se dalším příkazem.

    Když je podmínka prázdná, vyhodnotí se jako pravdivá -> nekonečný cyklus.

    Tělo cyklu tvoří jediný příkaz - silně doporučuji používat vždy blokový příkaz.
    */

    printh(
      "\n"
      KOD(
      " do <1> while (<2>);\n"
      )
      "\n"
      "- Vykonává posloupnost kroků v pořadí: <1>, <2>, <1>, <2>, ...\n"
      "- Tělo <1> se vykoná vždy nejméně jednou.\n"
      "- Tělo <1> může být prázdný příkaz (" KK(";") " nebo " KK("{}") "), nelze jej ale vynechat.\n"
      "\n"
      "- Podmínka <2> nesmí být prázdná. Musí jít o výraz, jehož hodnota jde\n"
      "  přetypovat na " KK("int") " a vyhodnotit jako logickou hodnotu.\n"
      "- Jakmile se podmínka <2> vyhodnotí jako nepravdivá, cyklus končí.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka cyklu do-while - 5 iterací")) return;
  if (pokracovat())
  {
    printh(
      "- Cyklus " KK("do-while") " od 0 do 4 => 5 iterací\n"
      "\n"
      KOD(
      "  int i = 0; // pokud by byla vyrobena uvnitř cyklu, nebyla by vidět v podmínce\n"
      "  do\n"
      "  {\n"
      "    printf(\"  %%d\", i);\n"
      "    i++;\n"
      "  } while (i < 5);\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );
    int i = 0; // pokud by byla vyrobena uvnitř cyklu, nebyla by vidět v podmínce
    do
    {
      printf("  %d", i);
      i++;
    } while (i < 5);
    /* Pozor! Sem  ^  skutečně patří středník, který ukončuje příkaz.

       Aby se to nepletlo s cyklem while, který má prázdné tělo, doporučuje se
       psát klíčové slovo while za zavírací složenou závorku na jednom řádku.
    */
    printg("\n\n");
  }

  if (navratNeboSekce("Ukázka - komunikace s uživatelem")) return;
  if (pokracovat())
  {
    printh(
      "- Cyklus " KK("do-while") " se často používá pro komunikaci s uživatelem,\n"
      "  když jej chcete otravovat tak dlouho, až zadá, co chcete.\n"
      "\n"
      KOD(
      "  short cislo;\n"
      "  do\n"
      "  {\n"
      "    printg(\"  => Zadej největší prvočíslo menší než 10: \");\n"
      "    int pocet = scanf(\"%%hd\", &cislo); // %%hd očekává hodnotu typu short\n"
      "    if (pocet != 1)\n"
      "      scanf(\"%%*s\"); // požere chybný vstup, když uživatel zadá písmena místo číslic\n"
      "  } while (cislo != 7);\n"
      )
    );
    short cislo;
    do
    {
      printg("  => Zadej největší prvočíslo menší než 10: ");
      int pocet = scanf("%hd", &cislo); // %hd očekává hodnotu typu short
      if (pocet != 1) scanf("%*s"); // požere chybný vstup, když uživatel zadá písmena místo číslic
    } while (cislo != 7);
    pozerKonecRadku(); // požere poslední Enter
    printg("  => Správně, je to hodnota 7.\n");
    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazReturn(void)
{
  zacatekKapitoly("Příkaz return");

  printh(
    "- Okamžitě ukončí podprogram v místě, kde je " KK("return") " zavolán.\n"
    "- Jde jej použít na více místech podprogramu.\n"
    "\n"
  );

  if (navratNeboSekce("Použití v procedurách")) return;
  if (pokracovat())
  {
    printh(
      "- Procedura je podprogram s prázdným návratovým typem " KK("void") ".\n"
      "- Příkaz " KK("return") " není nutný => procedura skončí na konci svého těla.\n"
      "- Pokud se někde použije, pak bez argumentu.\n"
      "\n"
      KOD(
      "    return;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Použití ve funkcích")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce je podprogram s konkrétním návratovým typem.\n"
      "- Použití příkazu " KK("return") " je povinné alespoň na konci.\n"
      "- Lze jej použít i kdekoli jinde v těle, kde lze použít příkaz.\n"
      "- Příkaz " KK("return") " musí mít argument typově kompatibilní s návratovým typem.\n"
      "\n"
      KOD(
      "    return hodnota;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Mrtvý kód")) return;
  if (pokracovat())
  {
    printh(
      "- Kód za provedeným příkazem " KK("return") " se nikdy neprovede, proto se mu říká\n"
      "  mrtvý kód.\n"
      "\n"
      KOD(
      "  return;\n"
      "  printf(\"Toto je mrtvý kód!\\n\");\n"
      )
      "\n"
    );
  }

  konecKapitoly(__func__);

  return; // V této proceduře je tento příkaz zbytečný.
  /* Kód bezprostředně za příkazem return je tzv. mrtvý kód.
     Takový kód se nikdy nemá šanci vykonat. Při zapnuté optimalizaci překladač
     takový kód pravděpodobně nezahrne do výsledného programu.
  */
  printg("Toto je mrtvý kód!\n");

}





////////////////////////////////////////////////////////////////////////////////
void prikazBreak(void)
{
  zacatekKapitoly("Příkaz break");

  printh(
    "- Ukončí běh cyklu nebo příkazu " KK("switch") ".\n"
    "- Program pokračuje dalšími příkazy za přerušeným cyklem nebo přepínačem.\n"
    "\n"
  );

  /* Příkaz break lze použít jen uvnitř cyklu nebo příkazu switch.

  Příkaz break okamžitě ukončí tělo příkazu, v němž se nachází.

  Pozor u vnořených cyklů - ukončí se jen ten cyklus, ve kterém se příkaz break
  zavolal. Z vnitřního cyklu nejde ukončit vnější cyklu. (ale jde - viz goto)
  while (podmínka) // vnější cyklus
  {
    while (podmínka) // vnitřní cyklus
    {
      ...
      break; // ukončí vnitřní cyklus, skočí do těla vnějšího cyklu, který dál pokračuje
      ...
    }
  }
  */

  if (navratNeboSekce("Ukázka - nekonečný cyklus")) return;
  if (pokracovat())
  {
    printh(
      "- Příkazem " KK("break") " jde ukončit každý cyklus.\n"
      "- Často se používá v \"nekonečných cyklech\".\n"
      "  - Taková konstrukce má smysl jen někdy - např. při zpracování událostí\n"
      "    v GUI aplikacích.\n"
      "  - V běžných algoritmech jde spíš o programátorskou chybu.\n"
      "\n"
      KOD(
      "  int i = 5;\n"
      "  while (1) // \"nekonečný\" cyklus\n"
      "  {\n"
      "    printf(\"All work and no play makes Jack a dull boy.\\n\");\n"
      "    if (--i == 0) // Podmínka konce cyklu uvnitř těla je většinou prasárna!\n"
      "    {\n"
      "      printf(\"break\\n\");\n"
      "      break;\n"
      "    }\n"
      "  }\n"
      )
      "\n"
    );
    int i = 5;
    while (1) // "nekonečný" cyklus
    {
      printf("  => All work and no play makes Jack a dull boy.\n");
      if (--i == 0) // podmínka konce cyklu uvnitř těla je většinou prasárna!
      {
        printf("  => break\n");
        break;
      }
    }

    printg("\n");
  }

  if (navratNeboSekce("Pozor na vnořené cykly!")) return;
  if (pokracovat())
  {
    printh(
      "- Příkaz " KK("break") " přerušuje jen ten cyklus, ve kterém je použit.\n"
      "- Příkaz " KK("break") " ve vnitřním cyklu nepřeruší vnější cyklus.\n"
      "- Pro skok z vícenásobného cyklu se dá použít příkaz " KK("goto") " - viz další sekce.\n"
      "\n"
      KOD(
      "  for (int i = 1; i <= 3; ++i)\n"
      "  {\n"
      "    printf(\"%%d. vnější iterace před\\n\", i);\n"
      "    for (int j = 1; j < 1000000; ++j)\n"
      "    {\n"
      "      printf(\"  %%d. vnitřní iterace\\n\", j);\n"
      "      if (j == 2)\n"
      "      {\n"
      "        printf(\"  break\\n\");\n"
      "        break;\n"
      "      }\n"
      "    }\n"
      "    printf(\"%%d. vnější iterace po\\n\", i);\n"
      "  }\n"
      )
      "\n"
    );

    for (int i = 1; i <= 3; ++i)
    {
      printg("  => %d. vnější iterace před\n", i);
      for (int j = 1; j < 1000000; ++j)
      {
        printg("  =>   %d. vnitřní iterace\n", j);
        if (j == 2)
        {
          printg("  =>   break\n");
          break;
        }
      }
      printg("  => %d. vnější iterace po\n", i);
    }
    printh("\n");
  }

  konecKapitoly(__func__);
}




////////////////////////////////////////////////////////////////////////////////

void prikazContinue(void)
{
  zacatekKapitoly("Příkaz continue");

  printh(
    "- Přeruší aktuální iteraci cyklu. Cyklus pak může pokračovat další iterací.\n"
    "- Příkaz jde použít pouze v těle nějakého cyklu.\n"
    "\n"
  );

  if (navratNeboSekce("Ukázka použití v cyklu while")) return;
  if (pokracovat())
  {

    printh(
      "- V cyklech " KK("while") " a " KK("do-while") " příkaz skočí rovnou na vykonání podmínky.\n"
      "\n"
      KOD(
      "  int i = -1;\n"
      "  //     continue skočí sem\n"
      "  //     ||\n"
      "  //     \\/\n"
      "  while (i < 10)\n"
      "  {\n"
      "    ++i;\n"
      "    if (i %% 2 == 0) continue;\n"
      "    printf(\"  %%d\", i);\n"
      "  }\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );
    int i = -1;
    //     continue skočí sem
    //     ||
    //     \/
    while (i < 10)
    {
      ++i; // na konci cyklu by to vedlo k zacyklení => zkuste to krokovat v debuggeru
      if (i % 2 == 0) continue;
      printf("  %d", i);
    }
    printh(
      "\n"
      "\n"
    );
  }

  if (navratNeboSekce("Použití v cyklu for")) return;
  if (pokracovat())
  {
    printh(
      "- V cyklu " KK("for") " příkaz skočí na vyhodnocení posledního výrazu v hlavičce.\n"
      "\n"
      KOD(
      "  //       continue skočí sem\n"
      "  //                      ||\n"
      "  //                      \\/\n"
      "  for (int i = 0; i < 10; ++i)\n"
      "  {\n"
      "    if (i %% 2 == 0) continue;\n"
      "    printf(\"  %%d\", i);\n"
      "  }\n"
      "  printf(\"\\n\");\n"
      )
      "\n"
      "  =>"
    );
    //       continue skočí sem
    //                      ||
    //                      \/
    for (int i = 0; i < 10; ++i)
    {
      if (i % 2 == 0) continue;
      printg("  %d", i);
    }
    printh("\n");
    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazGoto(void)
{
  zacatekKapitoly("Příkaz goto");

  printh(
    "- NEBEZPEČNÉ! NEPOUŽÍVAT!\n"
    "- Příkaz skočí na řádek kódu označený návěštím.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  goto návěští;\n"
      "\n"
      "návěští:\n"
      "  příkaz;\n"
      )
      "\n"
      "- Návěští nevykovává žádnou činnost. Je to jen identifikátor označující\n"
      "  nějaký řádek kódu, kam má příkaz " KK("goto") " skočit.\n"
      "- Za návěštím musí vždy být nějaký příkaz.\n"
      "\n"
      "- Příkaz " KK("goto") " nesmí skákat mimo tělo aktuální funkce.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Skok z vnitřního cyklu")) return;
  if (pokracovat())
  {

    /* Příkaz goto nikdy nepoužívejte! Vždy to jde bez něj. Může způsobit hrozné
    chyby.

    Jediným smysluplným využitím je ukončení vnořených cyklů.

    Používá se to takto:

    goto návěští;
  návěští:

    Návěští je identifikátor ukončený dvojtečkou, který se může nacházet kdekoli
    v rámci funkce, kde je goto použito. Nejde vyskočit ven z funkce do jiné
    funkce. Návěští je lokální uvnitř své funkce.
    */

    printh(
      "- Jediné ospravedlnitelné použití\n"
      "- Slouží k ukončení vícenásobně vnořeného cyklu.\n"
      "- Dá se nahradit pomocnou funkcí, kde cyklus ukončí " KK("return") ".\n"
      "\n"
      KOD(
      "  while (1) // vnější cyklus\n"
      "  {\n"
      "    printf(\"vnější...\\n\");\n"
      "    while (1) // vnitřní cyklus\n"
      "    {\n"
      "      printf(\"  vnitřní...\\n\");\n"
      "      printf(\"  goto konecCyklu;\\n\");\n"
      "      goto konecCyklu; // ukončí vnitřní i vnější cyklus\n"
      "    }\n"
      "  }\n"
      "konecCyklu:\n"
      "  printf(\"Tady je návěští konecCyklu.\\n\");\n"
      )
      "\n"
    );
    while (1) // vnější cyklus
    {
      printg("  =>vnější...\n");
      while (1) // vnitřní cyklus
      {
        printg("  =>  vnitřní...\n");
        printg("  =>  goto konecCyklu;\n");
        goto konecCyklu; // ukončí vnitřní i vnější cyklus
      }
    }
  konecCyklu:
    // poznámka: Návěští se nesmí ocitnout na konci bloku. Pokud je tam potřeba, je třeba za něj napsat prázdný příkaz {}.
    printg("  =>Tady je návěští konecCyklu.\n");


    printg("\n");
  }

  if (navratNeboSekce("Přeskočená inicializace")) return;
  if (pokracovat())
  {
    printh(
      "- Pozor! Pokud " KK("goto") " přeskočí inicializaci proměnné, bude v ní\n"
      "  neinicializovaná hodnota.\n"
      "\n"
      KOD(
      "  goto tisk;\n"
      "  int cislo = 10;\n"
      "tisk:\n"
      "  printf(\"Přeskočená inicializace: cislo = %%d\\n\", cislo);\n"
      )
      "\n"
      "  =>"
    );

// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
    /* Tady se sice proměnná cislo vyrobí, ale dojde k přeskočení inicializace! */
    goto tisk;
    int cislo = 10; // překladač hlásí varování před neinicializovanou hodnotou
  tisk:
    printg("Přeskočená inicializace: cislo = %d\n", cislo);
#pragma GCC diagnostic pop

    printh("\n");
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void prikazDefinicePromenne(void)
{
  zacatekKapitoly("Příkaz definice proměnné");

  printh(
    "- Když se ve funkci deklaruje proměnná, je to zároveň její definice.\n"
    "- Definice proměnné je příkaz k vyhrazení místa v paměti o velikosti,\n"
    "  která je dána použitým datovým typem.\n"
    "- Proměnná musí být vždy nejprve vyrobena, aby mohla být používána.\n"
    "\n"
    "- Definice proměnné je často spojena s inicializací.\n"
    "- Datové typy pole a struktura mají pro inicializaci během definice\n"
    "  speciální syntaktické prostředky. Viz kapitoly o polích a strukturách.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  typ identifikátor;\n"
      "  typ identifikátor = hodnota;\n"
      "  typ id1, id2, id3;\n"
      "  typ id1 = hodnota, id2, id3 = hodnota;\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Deklarace externí proměnné")) return;
  if (pokracovat())
  {
    printh(
      "- Používá se pro vytváření globálních proměnných a konstant v modulech.\n"
      "- V hlavičkovém souboru se proměnná deklaruje pomocí klíčového slova " KK("extern") ".\n"
      "- Ve zdrojovém souboru modulu se pak proměnná definuje a inicializuje.\n"
      "- Řeší to problém potenciální vícenásobné definice proměnné ve více modulech.\n"
      "\n"
      "- Pozor! Veřejné globální proměnné jsou prasárna. Nepoužívat!\n"
      "- Tohle má význam prakticky jen u konstantních proměnných.\n"
      "\n"
      PODSEKCE("Syntaxe")
      "\n"
      KOD(
      "  extern int hodnota;         // v modul.h\n"
      "  extern const int KONSTANTA; // v modul.h\n"
      "\n"
      "  int hodnota = 4;            // v modul.c\n"
      "  const int KONSTANTA = 17;   // v modul.c\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka definice proměnné")) return;
  if (pokracovat())
  {
    printh(
      "- Tady proměnná cislo ještě neexistuje.\n"
      "\n"
      KOD(
      "  printf(\"cislo = %%hd\\n\", cislo); // syntaktická chyba \n"
      )
    );
  //  printg("cislo = %hd\n", cislo); // odkomentuj to a zkus přeložit

    printh(
      "\n"
      "- Vyrábím proměnnou cislo.\n"
      "\n"
      KOD(
      "  short cislo = 10;\n"
      )
    );
    short cislo = 10;

    printh(
      "\n"
      "- Teď je proměnná cislo vyrobená.\n"
      "\n"
      KOD(
      "  printf(\"cislo = %%hd\\n\", cislo);\n"
      "  printf(\"Zabírá v paměti %%zu bajty.\\n\", sizeof(cislo));\n"
      )
      "\n"
      "  =>cislo = %hd\n"
      "  =>Zabírá v paměti %zu bajty.\n"
      "\n"
      , cislo
      , sizeof(cislo)
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
void vyrazovyPrikaz(void)
{
  zacatekKapitoly("Výrazy a výrazové příkazy");

  printh(
    "- Výrazy jsou konstrukce, které mají výsledek. Cílem je získat tuto hodnotu.\n"
    "- Příkazy mají za úkol provést kód. Výsledkem je činnost, ne hodnota.\n"
    "- Z každého výrazu se v jazyce C dá vyrobit příkaz, když se ukončí středníkem.\n"
    "\n"
  );

  /* Výraz - kód, jehož výsledkem je hodnota určitého typu.

  Tuto hodnotu jde někde použít, např. v jiném výrazu (výrazy mohou mít
  podvýrazy), jde ho přiřadit do proměnné nebo použít jako argument podprogramu.

  Může jít o aritmetický výraz (např. a + b*c), přístupový výraz (indexování:
  pole[index], přístup ke složce struktury: struktura.slozka, přístup ke složce
  struktury odkazované ukazatelem pstruktura->slozka), ukazatelové výrazy (*ptr,
  &prom), a jiné.

  L-hodnota - výraz, který reprezentuje paměťovou buňku - jde do něj přiřadit
  hodnotu => může se vyskytovat na levé straně přiřazovacího operátoru.
  Příklady: promenna, pole[i], *ptr, struktura.slozka, pstruktura->slozka

  P-hodnota - výraz, který nereprezentuje paměťovou buňku - jeho výsledkem je
  pouze hodnota nějakého typu => může se vyskytovat pouze na pravé straně
  přiřazovacího příkazu.
  Příklady: konstanta, 123, a + 5, &promenna

  Pozor! Název typu (int, float, struct jmeno, ...) není ani jedno z toho!
  */

  if (navratNeboSekce("Výraz")) return;
  if (pokracovat())
  {
    int a = 7, b = 10;
    printh(
      "- Kód, jehož výsledkem je hodnota určitého typu.\n"
      "- Tuto hodnotu jde někde použít, např.\n"
      "  - v podvýrazu,\n"
      "  - jako argument podprogramu,\n"
      "  - jde ji přiřadit do proměnné.\n"
      "\n"
      "- Tvoří jej operátory a operandy, které mohou být opět výrazy.\n"
      "- Platí, že operátory mají své priority (např. násobení má přednost před\n"
      "  sčítáním), které jde změnit pomocí závorek.\n"
      "- Tabulky s prioritami operátorů jde najít na webu.\n"
      "- Priority jsou navrženy přirozeně tak, aby nebylo potřeba moc závorkovat.\n"
      "  => Ale v případě pochybností závorkuj.\n"
      "\n"
      KOD(
      "  int a = %d, b = %d;\n"
      "  a + b"
      )
      "\t\t=> %d\n"
      KOD(
      "  a + b * 3"
      )
      "\t=> %d\n"
      KOD(
      "  (a + b) * 3"
      )
      "\t=> %d\n"

      "\n"
      , a, b
      , a + b
      , a + b * 3
      , (a + b) * 3
    );
  }

  if (navratNeboSekce("L-hodnota")) return;
  if (pokracovat())
  {
    printh(
      "- Anglicky se nazývá L-value.\n"
      "- Výraz reprezentující paměťovou buňku - proměnnou.\n"
      "- Jde do něj přiřadit hodnotu.\n"
      "- Může se vyskytovat na levé i pravé straně přiřazovacího příkazu.\n"
      "- Příklady\n"
      KOD(
      "  promenna\n"
      "  pole[i]\n"
      "  *ukazatel\n"
      "  struktura.slozka\n"
      "  pstruktura->slozka\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("P-hodnota")) return;
  if (pokracovat())
  {
    printh(
      "- Anglicky se nazývá R-value.\n"
      "- Je to výraz, který má výsledek určitého datového typu.\n"
      "- Nereprezentuje paměťovou buňku.\n"
      "- Nejde do něj přiřadit hodnotu.\n"
      "- Může se vyskytovat pouze na pravé straně přiřazovacího příkazu.\n"
      "- Příklady\n"
      KOD(
      "  KONSTANTA\n"
      "  123\n"
      "  (a + b)/2\n"
      "  &promenna\n"
      )
      "\n"
    );
  }

  if (navratNeboSekce("Příkaz")) return;
  if (pokracovat())
  {
    printh(
      "- Pokyn pro počítač k provedení nějaké akce.\n"
      "- Cílem je provedení kódu, ne získání hodnoty.\n"
      "\n"
      KOD(
      "  printf(\"Toto je příkaz.\\n\");\n"
      )
      "\n"
      "  =>"
    );
    printg("Toto je příkaz.\n");

    printg("\n");
  }

  if (navratNeboSekce("Výrazový příkaz")) return;
  if (pokracovat())
  {
    /* Z každého výrazu jde udělat příkaz tím, že se za něj zapíše středník.
    Hodnota takového výrazu se pak zahodí. Často se to využívá u funkcí, jejichž
    výsledek není potřeba, ale provedení jejich kódu ano. Funkce se prostě
    zavolá a výsledek se nikam nepřiřadí. Věděli jste, že printf je také funkce,
    která vrací číselný výsledek - počet zapsaných znaků? Tento údaj se skoro
    nikdy nepoužívá, protože se funkce printf používá jako příkaz.

    Součástí syntaxe příkazů v jazyce C jsou středníky. Středník ukončuje příkaz.

    Jedinou výjimkou je blokový příkaz {}, za nímž se středník nepíše.
    */
    int a = 1, b = 2;
    printh(
      "- Výraz ukončený středníkem je příkaz.\n"
      "- Může mít smysl kvůli svým vedlejším efektům - například funkce " KK("printf") "\n"
      "  má sice návratovou hodnotu, ale často se ignoruje - nikam se nepřiřadí.\n"
      "  => Funkce se zavolá jako procedura, přesto to má smysl.\n"
      "\n"
      KOD(
      "  1 + 1; // příkaz bez efektu - výsledná hodnota se zahodí\n"
      "  a / b;\n"
      "  printf(\"Howdy-doodly-do, how's it going?\\n\");\n"
      )
      "\n"
    );

// Tohle chci demonstrovat, tak dočasně vypínám hlídání této chyby (kvůli -Werror).
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-value"
    1 + 1;  // toto je příkaz bez efektu, tj. nic nedělá - překladač hlásí varování
    a / b;  // toto je příkaz bez efektu, tj. nic nedělá - překladač hlásí varování
    printf("  =>Howdy-doodly-do, how's it going?\n");
#pragma GCC diagnostic pop

    printg("\n");
  }

  if (navratNeboSekce("Prázdný příkaz")) return;
  if (pokracovat())
  {
    /* Samotný středník představuje prázdný příkaz:  {} a ; jsou totožné příkazy.
    Prázdný příkaz se používá například v cyklech, kde je potřeba prázdné tělo.
    Z důvodu lepší čitelnosti je doporučeno používat {} místo ;.

    Pozor! Cyklus, za jehož hlavičkou je středník znamená cyklus s prázdným tělem!
    i = 0;
    while (i < 10);   <- Tady končí cyklus s prázdným tělem! Zacyklí se!
    {                 <- Tady začíná další blokový příkaz, nikoli tělo cyklu!
      i++;
    }
    */
    printh(
      "- Používá se například v cyklech.\n"
      "- Přehlednější je varianta se závorkami.\n"
      "\n"
      KOD(
      "  ;   \n"
      "  {}\n"
      )
      "\n"
    );
    ;     // prázdný příkaz
    {}    // prázdný příkaz
  }

  if (navratNeboSekce("Cyklus s prázdným tělem")) return;
  if (pokracovat())
  {
    printh(
      "- Používá se často při zpracování souborů a datových proudů.\n"
      "\n"
      KOD(
      "  printf(\"Napiš posloupnost mezer, slovo a stiskni Enter\\n\");\n"
      "  int c;\n"
      "  // Cyklus přeskočí posloupnost mezer.\n"
      "  while ( ( c = getchar() ) == ' ') {}  // prázdné tělo cyklu\n"
      "  printf(\"První znak, co není mezera má ASCII kód %%d, je to znak '%%c'.\\n\", c, c);\n"
      )
      "\n"
    );
    printg("  =>Napiš posloupnost mezer, slovo a stiskni Enter\n  =>");
    int c;
    // Cyklus přeskočí posloupnost mezer.
    while ( ( c = getchar() ) == ' ') {}  // prázdné tělo cyklu
    printg("  =>První znak, co není mezera má ASCII kód %d, je to znak '%c'.\n", c, c);

    if (c != '\n')
      pozerKonecRadku();

    printg("\n");
  }

  if (navratNeboSekce("Přiřazovací výraz")) return;
  if (pokracovat())
  {
    /* V jazyce C je přiřazení výrazem!
    Paradoxně je to P-hodnota, protože výsledkem výrazu a=10 je hodnota 10,
    která jde dále používat, například znovu přiřadit: b = a = 10.

    Příkaz se z přiřazení udělá tím, že se ukončí středníkem.
    */
    printh(
      "- V C je přiřazení výrazem jehož výsledkem je přiřazovaná hodnota.\n"
      "- Vyhodnocuje se zprava doleva, tedy nejdříve výraz vpravo od operátoru " KK("=") ",\n"
      "  teprve pak se přiřadí výsledek.\n"
      "\n"
      KOD(
      "  a = b = 111;\n"
      )
      "\n"
    );
    int a, b;
    a = b = 111;
    printg("  => a: %d\n", a);
    printg("  => b: %d\n", b);
    printh(
      "\n"
      "- Přiřazení lze použít i uvnitř výrazu.\n"
      "- Přiřazení má velmi malou prioritu => běžné operátory mají přednost\n"
      "  => provádí se až jako poslední.\n"
      "\n"
      KOD(
      "  //   Toto je přiřazení uvnitř podmínky cyklu.\n"
      "          _______/\\________\n"
      "  while ( ( c = getchar() ) == ' ') {}\n"
      )
      "\n"
      "- Kvůli prioritám operátorů je zde nutné dát přiřazení do závorek.\n"
      "- Bez závorek by se do " KK("c") " přiřadil výsledek porovnání operátorem " KK("==") ".\n"
      "\n"
    );


    /*  Toto je přiřazení uvnitř podmínky cyklu.
            _______/\________
    while ( ( c = getchar() ) == ' ') {}

    Musí být v závorkách, jinak by se nejprve provedlo porovnání a až tato
    logická hodnota by se přiřadila do proměnné c.
    */
  }

  konecKapitoly(__func__);
}


